@@ -595,4 +595,70 @@ setMethod("show",
           }
 )
 
+#--------------------------------------------------------------------------------------------
 
+setMethod("show",
+          "PIVW",
+          function(object){
+
+             Interval_type <- paste(100*(1-object@Alpha), "% CI", sep = "")
+             Statistic <- c("Method", "Estimate", "Std Error", Interval_type, "", "p-value","Condition")
+
+             blk = rep("", length(object@CILower)-1)
+             Value <- c("pIVW", blk,
+                        decimals(object@Estimate, 3), blk,
+                        decimals(object@StdError,3), blk,
+                        paste(decimals(object@CILower, 3), ",", sep = ""),
+                        decimals(object@CIUpper, 3),
+                        decimals(object@Pvalue, 3), blk,
+                        decimals(object@Condition, 3), blk)
+
+             output.table <- data.frame(matrix(Value, nrow = length(object@CILower), byrow=FALSE))
+             colnames(output.table) <- Statistic
+
+             cat("\nPenalized inverse-variance weighted method\n\n")
+             cat("Over dispersion:", object@Over.dispersion, "\n")
+             cat("Bootstrapping Fieller:", object@Boot.Fieller, "\n")
+             cat("Penalty parameter (lambda):", object@Lambda, "\n")
+             cat("IV selection threshold (delta):", round(object@Delta,3), "\n")
+             cat("Number of variants :", object@SNPs, "\n")
+
+
+             cat("------------------------------------------------------------------\n")
+             print(output.table, quote = F, row.names = FALSE, justify= "left")
+             cat("------------------------------------------------------------------\n")
+          }
+)
+
+#--------------------------------------------------------------------------------------------
+
+setMethod("show",
+          "MVMRcML",
+          function(object){
+            if(object@DP){
+              Method = 'MVMRcML-DP'
+              n_eff_B = object@eff_DP_B
+            }else{
+              Method = 'MVMRcML-BIC'
+            }
+            Interval_type <- paste(100*(1-object@Alpha), "% CI", sep = "")
+            Statistic <- c("Exposure", "Estimate", "Std Error", Interval_type, "", "p-value")
+            
+            Value <- cbind(object@Exposure, decimals(object@Estimate, 3), decimals(object@StdError,3),
+                           paste(decimals(object@CILower, 3), ",", sep = ""), decimals(object@CIUpper,3),
+                           decimals(object@Pvalue, 3))
+            output.table <- data.frame(matrix(Value, nrow = length(object@Exposure)))
+            colnames(output.table) <- Statistic
+            
+            cat("\nMultivariable MRcML method \n\n")
+            cat("Number of variants :", object@SNPs, "\n")
+            cat("Results for: ",Method, "\n")
+            if(object@DP){
+              cat("Number of data perturbations with successful convergence: ",n_eff_B, "\n")
+            }
+            cat("------------------------------------------------------------------\n")
+            print(output.table, quote = F, row.names = FALSE, justify = "left")
+            cat("------------------------------------------------------------------\n")
+            
+          }
+         )
